"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""


from keras_core.src.initializers import deserialize
from keras_core.src.initializers import get
from keras_core.src.initializers import serialize
from keras_core.src.initializers.constant_initializers import Constant
from keras_core.src.initializers.constant_initializers import Constant as constant
from keras_core.src.initializers.constant_initializers import Identity
from keras_core.src.initializers.constant_initializers import Identity as IdentityInitializer
from keras_core.src.initializers.constant_initializers import Identity as identity
from keras_core.src.initializers.constant_initializers import Ones
from keras_core.src.initializers.constant_initializers import Ones as ones
from keras_core.src.initializers.constant_initializers import Zeros
from keras_core.src.initializers.constant_initializers import Zeros as zeros
from keras_core.src.initializers.initializer import Initializer
from keras_core.src.initializers.random_initializers import GlorotNormal
from keras_core.src.initializers.random_initializers import GlorotNormal as glorot_normal
from keras_core.src.initializers.random_initializers import GlorotUniform
from keras_core.src.initializers.random_initializers import GlorotUniform as glorot_uniform
from keras_core.src.initializers.random_initializers import HeNormal
from keras_core.src.initializers.random_initializers import HeNormal as he_normal
from keras_core.src.initializers.random_initializers import HeUniform
from keras_core.src.initializers.random_initializers import HeUniform as he_uniform
from keras_core.src.initializers.random_initializers import LecunNormal
from keras_core.src.initializers.random_initializers import LecunNormal as lecun_normal
from keras_core.src.initializers.random_initializers import LecunUniform
from keras_core.src.initializers.random_initializers import LecunUniform as lecun_uniform
from keras_core.src.initializers.random_initializers import OrthogonalInitializer
from keras_core.src.initializers.random_initializers import OrthogonalInitializer as Orthogonal
from keras_core.src.initializers.random_initializers import OrthogonalInitializer as orthogonal
from keras_core.src.initializers.random_initializers import RandomNormal
from keras_core.src.initializers.random_initializers import RandomNormal as random_normal
from keras_core.src.initializers.random_initializers import RandomUniform
from keras_core.src.initializers.random_initializers import RandomUniform as random_uniform
from keras_core.src.initializers.random_initializers import TruncatedNormal
from keras_core.src.initializers.random_initializers import TruncatedNormal as truncated_normal
from keras_core.src.initializers.random_initializers import VarianceScaling
from keras_core.src.initializers.random_initializers import VarianceScaling as variance_scaling
