"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""


from keras_core.src.losses import deserialize
from keras_core.src.losses import get
from keras_core.src.losses import serialize
from keras_core.src.losses.loss import Loss
from keras_core.src.losses.losses import BinaryCrossentropy
from keras_core.src.losses.losses import BinaryFocalCrossentropy
from keras_core.src.losses.losses import CategoricalCrossentropy
from keras_core.src.losses.losses import CategoricalFocalCrossentropy
from keras_core.src.losses.losses import CategoricalHinge
from keras_core.src.losses.losses import CosineSimilarity
from keras_core.src.losses.losses import Hinge
from keras_core.src.losses.losses import Huber
from keras_core.src.losses.losses import KLDivergence
from keras_core.src.losses.losses import LogCosh
from keras_core.src.losses.losses import MeanAbsoluteError
from keras_core.src.losses.losses import MeanAbsolutePercentageError
from keras_core.src.losses.losses import MeanSquaredError
from keras_core.src.losses.losses import MeanSquaredLogarithmicError
from keras_core.src.losses.losses import Poisson
from keras_core.src.losses.losses import SparseCategoricalCrossentropy
from keras_core.src.losses.losses import SquaredHinge
from keras_core.src.losses.losses import binary_crossentropy
from keras_core.src.losses.losses import binary_focal_crossentropy
from keras_core.src.losses.losses import categorical_crossentropy
from keras_core.src.losses.losses import categorical_focal_crossentropy
from keras_core.src.losses.losses import categorical_hinge
from keras_core.src.losses.losses import cosine_similarity
from keras_core.src.losses.losses import hinge
from keras_core.src.losses.losses import huber
from keras_core.src.losses.losses import kl_divergence
from keras_core.src.losses.losses import log_cosh
from keras_core.src.losses.losses import mean_absolute_error
from keras_core.src.losses.losses import mean_absolute_percentage_error
from keras_core.src.losses.losses import mean_squared_error
from keras_core.src.losses.losses import mean_squared_logarithmic_error
from keras_core.src.losses.losses import poisson
from keras_core.src.losses.losses import sparse_categorical_crossentropy
from keras_core.src.losses.losses import squared_hinge
