"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""


from keras_core.src.constraints import deserialize
from keras_core.src.constraints import get
from keras_core.src.constraints import serialize
from keras_core.src.constraints.constraints import Constraint
from keras_core.src.constraints.constraints import MaxNorm
from keras_core.src.constraints.constraints import MaxNorm as max_norm
from keras_core.src.constraints.constraints import MinMaxNorm
from keras_core.src.constraints.constraints import MinMaxNorm as min_max_norm
from keras_core.src.constraints.constraints import NonNeg
from keras_core.src.constraints.constraints import NonNeg as non_neg
from keras_core.src.constraints.constraints import UnitNorm
from keras_core.src.constraints.constraints import UnitNorm as unit_norm
