"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""


from keras_core.src.regularizers import deserialize
from keras_core.src.regularizers import get
from keras_core.src.regularizers import serialize
from keras_core.src.regularizers.regularizers import L1
from keras_core.src.regularizers.regularizers import L1 as l1
from keras_core.src.regularizers.regularizers import L1L2
from keras_core.src.regularizers.regularizers import L1L2 as l1_l2
from keras_core.src.regularizers.regularizers import L2
from keras_core.src.regularizers.regularizers import L2 as l2
from keras_core.src.regularizers.regularizers import OrthogonalRegularizer
from keras_core.src.regularizers.regularizers import OrthogonalRegularizer as orthogonal_regularizer
from keras_core.src.regularizers.regularizers import Regularizer
